@@ -166,13 +166,45 @@ mark_job_as_started(ScheduledBgwJob *sjob)
 }
 
 static void
-mark_job_as_ended(ScheduledBgwJob *sjob, JobResult res)
+mark_job_as_ended(ScheduledBgwJob *sjob, JobResult res, Jsonb *edata)
 {
 	Assert(sjob->may_need_mark_end);
-	ts_bgw_job_stat_mark_end(&sjob->job, res, NULL);
+	ts_bgw_job_stat_mark_end(&sjob->job, res, edata);
 	sjob->may_need_mark_end = false;
 }
 
+static ErrorData *
+makeJobErrorData(ScheduledBgwJob *sjob, JobResult res)
+{
+	ErrorData *edata = (ErrorData *) palloc0(sizeof(ErrorData));
+	edata->elevel = ERROR;
+	edata->sqlerrcode = ERRCODE_INTERNAL_ERROR;
+	edata->hint = NULL;
+
+	Assert(res != JOB_SUCCESS);
+
+	switch (res)
+	{
+		case JOB_FAILURE_TO_START:
+			edata->message = "failed to start job";
+			edata->detail = psprintf("Job %d (\"%s\") failed to start",
+									 sjob->job.fd.id,
+									 NameStr(sjob->job.fd.application_name));
+			break;
+		case JOB_FAILURE_IN_EXECUTION:
+			edata->message = "failed to execute job";
+			edata->detail = psprintf("Job %d (\"%s\") failed to execute.",
+									 sjob->job.fd.id,
+									 NameStr(sjob->job.fd.application_name));
+			break;
+		default:
+			pg_unreachable();
+			break;
+	}
+
+	return edata;
+}
+
 static void
 worker_state_cleanup(ScheduledBgwJob *sjob)
 {
@@ -224,11 +256,14 @@ worker_state_cleanup(ScheduledBgwJob *sjob)
 			 * Usually the job process will mark the end, but if the job gets
 			 * a signal (cancel or terminate), it won't be able to so we
 			 * should.
-			 * TODO: Insert a record in the job_errors table informing of this failure
-			 * Currently the SIGTERM case is not handled, there might be other cases as well
 			 */
 			elog(LOG, "job %d failed", sjob->job.fd.id);
-			mark_job_as_ended(sjob, JOB_FAILURE);
+			ErrorData *edata = makeJobErrorData(sjob, JOB_FAILURE_IN_EXECUTION);
+			mark_job_as_ended(sjob,
+							  JOB_FAILURE_IN_EXECUTION,
+							  ts_errdata_to_jsonb(edata,
+												  &sjob->job.fd.proc_schema,
+												  &sjob->job.fd.proc_name));
 		}
 		else
 		{
@@ -357,7 +392,12 @@ on_failure_to_start_job(ScheduledBgwJob *sjob)
 		/* restore the original next_start to maintain priority (it is unset during mark_start) */
 		if (sjob->next_start != DT_NOBEGIN)
 			ts_bgw_job_stat_set_next_start(sjob->job.fd.id, sjob->next_start);
-		mark_job_as_ended(sjob, JOB_FAILURE_TO_START);
+		ErrorData *edata = makeJobErrorData(sjob, JOB_FAILURE_TO_START);
+		mark_job_as_ended(sjob,
+						  JOB_FAILURE_TO_START,
+						  ts_errdata_to_jsonb(edata,
+											  &sjob->job.fd.proc_schema,
+											  &sjob->job.fd.proc_name));
 	}
 	scheduled_bgw_job_transition_state_to(sjob, JOB_STATE_SCHEDULED);
 	CommitTransactionCommand();
@@ -751,6 +791,17 @@ ts_bgw_scheduler_process(int32 run_for_interval_ms,
 
 	pgstat_report_activity(STATE_RUNNING, NULL);
 
+	/* If we are restoring or upgrading, don't schedule anything. Just
+	 * exit. */
+	if (ts_guc_restoring || IsBinaryUpgrade)
+	{
+		elog(LOG,
+			 "scheduler for database %u exiting since the database is restoring or upgrading",
+			 MyDatabaseId);
+		terminate_all_jobs_and_release_workers();
+		goto scheduler_exit;
+	}
+
 	/* txn to read the list of jobs from the DB */
 	StartTransactionCommand();
 	scheduled_jobs = ts_update_scheduled_jobs_list(scheduled_jobs, scheduler_mctx);
@@ -822,10 +873,12 @@ ts_bgw_scheduler_process(int32 run_for_interval_ms,
 		elog(WARNING, "bgw scheduler stopped due to shutdown_bgw guc");
 #endif
 
+scheduler_exit:
 	CHECK_FOR_INTERRUPTS();
 
 	wait_for_all_jobs_to_shutdown();
 	check_for_stopped_and_timed_out_jobs();
+	scheduled_jobs = NIL;
 }
 
 static void
