@@ -240,6 +240,7 @@ hypertable_modify_explain(CustomScanState *node, List *ancestors, ExplainState *
 		foreach (lc, chunk_dispatch_states)
 		{
 			ChunkDispatchState *cds = (ChunkDispatchState *) lfirst(lc);
+			state->batches_deleted += cds->batches_deleted;
 			state->batches_filtered += cds->batches_filtered;
 			state->batches_decompressed += cds->batches_decompressed;
 			state->tuples_decompressed += cds->tuples_decompressed;
@@ -251,6 +252,8 @@ hypertable_modify_explain(CustomScanState *node, List *ancestors, ExplainState *
 		ExplainPropertyInteger("Batches decompressed", NULL, state->batches_decompressed, es);
 	if (state->tuples_decompressed > 0)
 		ExplainPropertyInteger("Tuples decompressed", NULL, state->tuples_decompressed, es);
+	if (state->batches_deleted > 0)
+		ExplainPropertyInteger("Batches deleted", NULL, state->batches_deleted, es);
 }
 
 static CustomExecMethods hypertable_modify_state_methods = {
@@ -688,6 +691,31 @@ ExecModifyTable(CustomScanState *cs_node, PlanState *pstate)
 		if (pstate->ps_ExprContext)
 			ResetExprContext(pstate->ps_ExprContext);
 
+#if PG17_GE
+		/*
+		 * If there is a pending MERGE ... WHEN NOT MATCHED [BY TARGET] action
+		 * to execute, do so now --- see the comments in ExecMerge().
+		 */
+		if (node->mt_merge_pending_not_matched != NULL)
+		{
+			context.planSlot = node->mt_merge_pending_not_matched;
+
+			slot = ht_ExecMergeNotMatched(&context, node->resultRelInfo, cds, node->canSetTag);
+
+			/* Clear the pending action */
+			node->mt_merge_pending_not_matched = NULL;
+
+			/*
+			 * If we got a RETURNING result, return it to the caller.  We'll
+			 * continue the work on next call.
+			 */
+			if (slot)
+				return slot;
+
+			continue; /* continue with the next tuple */
+		}
+#endif
+
 		context.planSlot = ExecProcNode(subplanstate);
 
 		if (cds && cds->rri && operation == CMD_INSERT && cds->skip_current_tuple)
@@ -733,8 +761,15 @@ ExecModifyTable(CustomScanState *cs_node, PlanState *pstate)
 				if (operation == CMD_MERGE)
 				{
 					EvalPlanQualSetSlot(&node->mt_epqstate, context.planSlot);
-					ht_ExecMerge(&context, node->resultRelInfo, cds, NULL, node->canSetTag);
-					continue; /* no RETURNING support yet */
+					slot = ht_ExecMerge(&context,
+										node->resultRelInfo,
+										cds,
+										NULL,
+										NULL,
+										node->canSetTag);
+					if (slot)
+						return slot;
+					continue;
 				}
 #endif
 				elog(ERROR, "tableoid is NULL");
@@ -808,8 +843,15 @@ ExecModifyTable(CustomScanState *cs_node, PlanState *pstate)
 					if (operation == CMD_MERGE)
 					{
 						EvalPlanQualSetSlot(&node->mt_epqstate, context.planSlot);
-						ht_ExecMerge(&context, node->resultRelInfo, cds, NULL, node->canSetTag);
-						continue; /* no RETURNING support yet */
+						slot = ht_ExecMerge(&context,
+											node->resultRelInfo,
+											cds,
+											NULL,
+											NULL,
+											node->canSetTag);
+						if (slot)
+							return slot;
+						continue;
 					}
 #endif
 					elog(ERROR, "ctid is NULL");
@@ -839,9 +881,29 @@ ExecModifyTable(CustomScanState *cs_node, PlanState *pstate)
 			else if (AttributeNumberIsValid(resultRelInfo->ri_RowIdAttNo))
 			{
 				datum = ExecGetJunkAttribute(slot, resultRelInfo->ri_RowIdAttNo, &isNull);
+#if PG17_GE
+				if (isNull)
+				{
+					if (operation == CMD_MERGE)
+					{
+						EvalPlanQualSetSlot(&node->mt_epqstate, context.planSlot);
+						slot = ht_ExecMerge(&context,
+											node->resultRelInfo,
+											cds,
+											NULL,
+											NULL,
+											node->canSetTag);
+						if (slot)
+							return slot;
+						continue;
+					}
+					elog(ERROR, "wholerow is NULL");
+				}
+#else
 				/* shouldn't ever get a null result... */
 				if (isNull)
 					elog(ERROR, "wholerow is NULL");
+#endif
 
 				oldtupdata.t_data = DatumGetHeapTupleHeader(datum);
 				oldtupdata.t_len = HeapTupleHeaderGetDatumLength(oldtupdata.t_data);
@@ -915,7 +977,8 @@ ExecModifyTable(CustomScanState *cs_node, PlanState *pstate)
 				break;
 #if PG15_GE
 			case CMD_MERGE:
-				slot = ht_ExecMerge(&context, resultRelInfo, cds, tupleid, node->canSetTag);
+				slot =
+					ht_ExecMerge(&context, resultRelInfo, cds, tupleid, oldtuple, node->canSetTag);
 				break;
 #endif
 			default:
@@ -1545,7 +1608,12 @@ ExecInsert(ModifyTableContext *context, ResultRelInfo *resultRelInfo, TupleTable
 		 */
 		if (mtstate->operation == CMD_UPDATE)
 			wco_kind = WCO_RLS_UPDATE_CHECK;
-#if PG15_GE
+#if PG17_GE
+		else if (mtstate->operation == CMD_MERGE)
+			wco_kind = (mtstate->mt_merge_action->mas_action->commandType == CMD_UPDATE) ?
+						   WCO_RLS_UPDATE_CHECK :
+						   WCO_RLS_INSERT_CHECK;
+#elif PG15_GE
 		else if (mtstate->operation == CMD_MERGE)
 			wco_kind = (context->relaction->mas_action->commandType == CMD_UPDATE) ?
 						   WCO_RLS_UPDATE_CHECK :
