@@ -642,7 +642,7 @@ timescaledb_planner(Query *parse, const char *query_string, int cursor_opts,
 		context.rootquery = parse;
 		context.current_query = parse;
 
-		if (ts_extension_is_loaded())
+		if (ts_extension_is_loaded_and_not_upgrading())
 		{
 #ifdef USE_TELEMETRY
 			ts_telemetry_function_info_gather(parse);
@@ -663,7 +663,7 @@ timescaledb_planner(Query *parse, const char *query_string, int cursor_opts,
 			/* Call the standard planner */
 			stmt = standard_planner(parse, query_string, cursor_opts, bound_params);
 
-		if (ts_extension_is_loaded())
+		if (ts_extension_is_loaded_and_not_upgrading())
 		{
 			/*
 			 * Our top-level HypertableInsert plan node that wraps ModifyTable needs
@@ -1254,7 +1254,7 @@ apply_optimizations(PlannerInfo *root, TsRelType reltype, RelOptInfo *rel, Range
 static bool
 valid_hook_call(void)
 {
-	return ts_extension_is_loaded() && planner_hcache_exists();
+	return ts_extension_is_loaded_and_not_upgrading() && planner_hcache_exists();
 }
 
 static bool
@@ -1419,12 +1419,9 @@ timescaledb_get_relation_info_hook(PlannerInfo *root, Oid relation_objectid, boo
 				(type == TS_REL_CHUNK_CHILD) && IS_UPDL_CMD(query);
 			if (use_transparent_decompression && (is_standalone_chunk || is_child_chunk_in_update))
 			{
-				TimescaleDBPrivate *fdw_private = (TimescaleDBPrivate *) rel->fdw_private;
-				Assert(fdw_private->cached_chunk_struct == NULL);
-				fdw_private->cached_chunk_struct =
-					ts_chunk_get_by_relid(rte->relid, /* fail_if_not_found = */ true);
-				if (!ts_chunk_is_partial(fdw_private->cached_chunk_struct) &&
-					ts_chunk_is_compressed(fdw_private->cached_chunk_struct))
+				const Chunk *chunk = ts_planner_chunk_fetch(root, rel);
+
+				if (!ts_chunk_is_partial(chunk) && ts_chunk_is_compressed(chunk))
 				{
 					rel->indexlist = NIL;
 				}
@@ -1598,16 +1595,12 @@ timescaledb_create_upper_paths_hook(PlannerInfo *root, UpperRelationKind stage,
 	if (prev_create_upper_paths_hook != NULL)
 		prev_create_upper_paths_hook(root, stage, input_rel, output_rel, extra);
 
-	if (!ts_extension_is_loaded())
+	if (!ts_extension_is_loaded_and_not_upgrading())
 		return;
 
 	if (input_rel != NULL)
 		reltype = ts_classify_relation(root, input_rel, &ht);
 
-	if (ts_cm_functions->create_upper_paths_hook != NULL)
-		ts_cm_functions
-			->create_upper_paths_hook(root, stage, input_rel, output_rel, reltype, ht, extra);
-
 	if (output_rel != NULL)
 	{
 		/* Modify for INSERTs on a hypertable */
@@ -1624,23 +1617,19 @@ timescaledb_create_upper_paths_hook(PlannerInfo *root, UpperRelationKind stage,
 		}
 	}
 
-	if (!ts_guc_enable_optimizations || input_rel == NULL || IS_DUMMY_REL(input_rel))
-		return;
-
-	if (!involves_hypertable(root, input_rel))
-		return;
-
-	if (stage == UPPERREL_GROUP_AGG && output_rel != NULL)
+	if (stage == UPPERREL_GROUP_AGG && output_rel != NULL && ts_guc_enable_optimizations &&
+		input_rel != NULL && !IS_DUMMY_REL(input_rel) && involves_hypertable(root, input_rel))
 	{
 		if (parse->hasAggs)
 			ts_preprocess_first_last_aggregates(root, root->processed_tlist);
 
-		if (ts_guc_enable_chunkwise_aggregation)
-			ts_pushdown_partial_agg(root, ht, input_rel, output_rel, extra);
-
 		if (!partials_found)
 			ts_plan_add_hashagg(root, input_rel, output_rel);
 	}
+
+	if (ts_cm_functions->create_upper_paths_hook != NULL)
+		ts_cm_functions
+			->create_upper_paths_hook(root, stage, input_rel, output_rel, reltype, ht, extra);
 }
 
 static bool
