@@ -8,6 +8,7 @@
 #include <postgres.h>
 
 #include <access/htup_details.h>
+#include <access/tupdesc.h>
 #include <catalog/namespace.h>
 #include <catalog/pg_proc.h>
 #include <common/int.h>
@@ -21,6 +22,67 @@
 
 #include "compat/compat.h"
 
+/*
+ * Macro for debug messages that should *only* be present in debug builds but
+ * which should be removed in release builds. This is typically used for
+ * debug builds for development purposes.
+ *
+ * Note that some debug messages might be relevant to deploy in release build
+ * for debugging production systems. This macro is *not* for those cases.
+ */
+#ifdef TS_DEBUG
+#define TS_DEBUG_LOG(FMT, ...) elog(DEBUG2, "%s - " FMT, __func__, ##__VA_ARGS__)
+#else
+#define TS_DEBUG_LOG(FMT, ...)
+#endif
+
+#ifdef TS_DEBUG
+
+static inline const char *
+yes_no(bool value)
+{
+	return value ? "yes" : "no";
+}
+
+/* Convert datum to string using the output function. */
+static inline const char *
+datum_as_string(Oid typid, Datum value, bool is_null)
+{
+	Oid typoutput;
+	bool typIsVarlena;
+
+	if (is_null)
+		return "<NULL>";
+
+	getTypeOutputInfo(typid, &typoutput, &typIsVarlena);
+	return OidOutputFunctionCall(typoutput, value);
+}
+
+static inline const char *
+slot_as_string(TupleTableSlot *slot)
+{
+	StringInfoData info;
+	initStringInfo(&info);
+	appendStringInfoString(&info, "{");
+	for (int i = 0; i < slot->tts_tupleDescriptor->natts; i++)
+	{
+		Form_pg_attribute att = TupleDescAttr(slot->tts_tupleDescriptor, i);
+
+		if (att->attisdropped)
+			continue;
+		appendStringInfo(&info,
+						 "%s: %s",
+						 NameStr(att->attname),
+						 datum_as_string(att->atttypid, slot->tts_values[i], slot->tts_isnull[i]));
+		if (i + 1 < slot->tts_tupleDescriptor->natts)
+			appendStringInfoString(&info, ", ");
+	}
+	appendStringInfoString(&info, "}");
+	return info.data;
+}
+
+#endif /* TS_DEBUG */
+
 /*
  * Get the function name in a PG_FUNCTION.
  *
@@ -309,3 +371,9 @@ ts_datum_set_objectid(const AttrNumber attno, NullableDatum *datums, const Oid v
 	else
 		datums[AttrNumberGetAttrOffset(attno)].isnull = true;
 }
+
+extern TSDLLEXPORT void ts_get_rel_info_by_name(const char *relnamespace, const char *relname,
+												Oid *relid, Oid *amoid, char *relkind);
+extern TSDLLEXPORT void ts_get_rel_info(Oid relid, Oid *amoid, char *relkind);
+extern TSDLLEXPORT bool ts_is_hypercore_am(Oid amoid);
+extern TSDLLEXPORT Jsonb *ts_errdata_to_jsonb(ErrorData *edata, Name proc_schema, Name proc_name);
