@@ -632,7 +632,6 @@ select * from tidrangescan_test where time > '2023-02-12 00:00:00+02:40'::timest
 
 drop table tidrangescan_test;
 
-\set ON_ERROR_STOP 0
 \set VERBOSITY default
 set client_min_messages = WARNING;
 -- test creating a hypertable from table referenced by a foreign key fails with
@@ -643,5 +642,27 @@ create table test_schema.fk_child(
   id int,
   foreign key (time, id) references test_schema.fk_parent(time, id)
 );
+select create_hypertable ('test_schema.fk_child', 'time');
+\set ON_ERROR_STOP 0
 select create_hypertable ('test_schema.fk_parent', 'time');
 \set ON_ERROR_STOP 1
+
+-- create default indexes on chunks when migrating data
+CREATE TABLE test(time TIMESTAMPTZ, val BIGINT);
+CREATE INDEX test_val_idx ON test(val);
+INSERT INTO test VALUES('2024-01-01 00:00:00-03', 500);
+SELECT FROM create_hypertable('test', 'time', migrate_data=>TRUE);
+-- should return ALL indexes for hypertable and chunk
+SELECT * FROM test.show_indexes('test') ORDER BY 1;
+SELECT * FROM show_chunks('test') ch, LATERAL test.show_indexes(ch) ORDER BY 1, 2;
+DROP TABLE test;
+
+-- don't create default indexes on chunks when migrating data
+CREATE TABLE test(time TIMESTAMPTZ, val BIGINT);
+CREATE INDEX test_val_idx ON test(val);
+INSERT INTO test VALUES('2024-01-01 00:00:00-03', 500);
+SELECT FROM create_hypertable('test', 'time', create_default_indexes => FALSE, migrate_data=>TRUE);
+-- should NOT return default indexes for hypertable and chunk
+-- only user indexes should be returned
+SELECT * FROM test.show_indexes('test') ORDER BY 1;
+SELECT * FROM show_chunks('test') ch, LATERAL test.show_indexes(ch) ORDER BY 1, 2;
